package Catalyst::Plugin::Authentication::Store::DBIx::Class::Backend;

use strict;
use warnings;
use base qw/Class::Accessor::Fast/;

sub new {
    my ( $class, $config, $app ) = @_;

    ## figure out if we are overriding the default store user class.
    my $storeclass = $config->{'store_user_class'} || "Catalyst::Plugin::Authentication::Store::DBIx::Class::User";
    
    ## fields can be specified to be ignored during user location.  This allows
    ## authinfo to contain the user info required to find the user, as well as the password
    ## to try to match to, for example.  It can be added to by setting ignore_fields in the 
    ## authinfo hashref also. 
    $config->{'ignore_fields_in_find'} ||= [ 'password' ];
    push @{$config->{'ignore_fields_in_find'}}, ('searchargs', 'ignore_fields');
    
    ## make sure the store class is loaded.
    Catalyst::Utils::ensure_class_loaded( $storeclass );
    
    my $self = {};
    $self->{config} = $config;
    $self->{store_user_class} = $storeclass;
    
    #$self->{role_relation} ||= 'roles';
    #$self->{role_field} ||= 'role';

    bless $self, $class;
}

sub from_session {
    my ( $self, $c, $frozenuser ) = @_;

    return $frozenuser if ref $frozenuser;

    # this could be a lot better.  But for now it just assumes $frozenuser is an id and uses find_user
    # XXX: hits the database on every request?  Not good...
    return $self->find_user( { id => $frozenuser }, $c);
}

sub for_session {
    my ($self, $c, $user) = @_;
    
    return $user->for_session($c);
}

sub find_user {
    my ( $self, $authinfo, $c ) = @_;
    
    return $self->{'store_user_class'}->new($authinfo, $self->{config}, $c);
}


sub user_supports {
    # this can work as a class method
    shift->{'store_user_class'}->supports( @_ );
}

__PACKAGE__;

__END__

=head1 NAME

Catalyst::Plugin::Authentication::Store::DBIx::Class::Backend - A class to ...

=head1 VERSION

This documentation refers to version 0.01.

=head1 SYNOPSIS

use Catalyst::Plugin::Authentication::Store::DBIx::Class::Backend;

=head1 DESCRIPTION

The Catalyst::Plugin::Authentication::Store::DBIx::Class::Backend class implements ...

=head1 SUBROUTINES / METHODS

=head2 new (constructor)

Parameters:
    class
    config
    app

Insert description of constructor here...

=head2 from_session (method)

Parameters:
    c
    frozenuser

Insert description of method here...

=head2 for_session (method)

Parameters:
    c
    user

Insert description of method here...

=head2 find_user (method)

Parameters:
    authinfo
    c

Insert description of method here...

=head2 user_supports

Parameters:
    none

Insert description of subroutine here...

=head1 DEPENDENCIES

Modules used, version dependencies, core yes/no

strict

warnings

=head1 NOTES

...

=head1 BUGS AND LIMITATIONS

None known currently, please email the author if you find any.

=head1 SEE ALSO

L<Catalyst::Plugin::Authentication::Store::DBIC>, L<Catalyst::Plugin::Authentication>,
L<Catalyst::Plugin::Authorization::Roles>

=head1 AUTHOR

Jason Kuri (jk@domain.tld)

=head1 LICENCE

Copyright 2006 by Jason Kuri.

This software is free.  It is licensed under the same terms as Perl itself.

=cut
